@@ -286,12 +286,14 @@ class Resource:
         FORKLIFT_KONVEYOR_IO: str = "forklift.konveyor.io"
         INSTANCETYPE_KUBEVIRT_IO: str = "instancetype.kubevirt.io"
         HCO_KUBEVIRT_IO: str = "hco.kubevirt.io"
+        HELM_MARIADB_MMONTES_IO: str = "helm.mariadb.mmontes.io"
         HIVE_OPENSHIFT_IO: str = "hive.openshift.io"
         HOSTPATHPROVISIONER_KUBEVIRT_IO: str = "hostpathprovisioner.kubevirt.io"
         IMAGE_OPENSHIFT_IO: str = "image.openshift.io"
         IMAGE_REGISTRY: str = "registry.redhat.io"
         INTEGREATLY_ORG: str = "integreatly.org"
         K8S_CNI_CNCF_IO: str = "k8s.cni.cncf.io"
+        K8S_MARIADB_COM: str = "k8s.mariadb.com"
         K8S_V1_CNI_CNCF_IO: str = "k8s.v1.cni.cncf.io"
         KUBERNETES_IO: str = "kubernetes.io"
         KUBEVIRT_IO: str = "kubevirt.io"
